"""Backup operations for PostgreSQL via WAL-G and logical dumps."""

import re
from dataclasses import dataclass
from datetime import datetime
from typing import List, Optional

from rich.console import Console
from rich.progress import Progress, SpinnerColumn, TextColumn

from .config import Config
from .ssh import RemoteExecutor, SSHError

console = Console()


@dataclass
class BackupInfo:
    """Information about a backup."""

    name: str
    timestamp: datetime
    size: Optional[str] = None
    type: str = "unknown"
    database: Optional[str] = None


class WALGBackupManager:
    """Manager for WAL-G backup operations."""

    def __init__(self, config: Config):
        self.config = config

    def create_backup(self, executor: RemoteExecutor) -> bool:
        """
        Create a new WAL-G backup.

        Args:
            executor: SSH executor instance

        Returns:
            True if backup was successful, False otherwise
        """
        console.print("[blue]Creating WAL-G backup...[/blue]")

        if not executor.check_container_running(self.config.postgres_container):
            console.print(
                f"[red]PostgreSQL container '{self.config.postgres_container}' is not running[/red]"
            )
            return False

        command = (
            'su - postgres -c "WALG_DELTA_MAX_STEPS=0 '
            'envdir /etc/wal-g/env /usr/local/bin/wal-g backup-push /var/lib/postgresql/data"'
        )

        try:
            with Progress(
                SpinnerColumn(),
                TextColumn("[progress.description]{task.description}"),
                console=console,
                transient=True,
            ) as progress:
                progress.add_task("Creating backup...", total=None)

                stdout, stderr, exit_code = executor.docker_exec(
                    self.config.postgres_container, command
                )

                if exit_code == 0:
                    console.print("[green]✓ WAL-G backup created successfully[/green]")
                    return True
                else:
                    console.print("[red]✗ WAL-G backup failed[/red]")
                    if stderr:
                        console.print(f"[red]Error: {stderr}[/red]")
                    return False

        except SSHError as e:
            console.print(f"[red]✗ WAL-G backup failed: {e}[/red]")
            return False

    def list_backups(self, executor: RemoteExecutor) -> List[BackupInfo]:
        """
        List available WAL-G backups.

        Args:
            executor: SSH executor instance

        Returns:
            List of BackupInfo objects
        """
        if not executor.check_container_running(self.config.postgres_container):
            console.print(
                f"[red]PostgreSQL container '{self.config.postgres_container}' is not running[/red]"
            )
            return []

        command = "envdir /etc/wal-g/env /usr/local/bin/wal-g backup-list"

        try:
            stdout, stderr, exit_code = executor.docker_exec(
                self.config.postgres_container, command
            )

            if exit_code != 0:
                console.print("[red]Failed to list WAL-G backups[/red]")
                if stderr:
                    console.print(f"[red]Error: {stderr}[/red]")
                return []

            return self._parse_walg_backup_list(stdout)

        except SSHError as e:
            console.print(f"[red]Failed to list WAL-G backups: {e}[/red]")
            return []

    def _parse_walg_backup_list(self, output: str) -> List[BackupInfo]:
        """Parse WAL-G backup list output."""
        backups = []
        lines = output.strip().split("\n")

        for line in lines:
            line = line.strip()
            if not line or line.startswith("name") or line.startswith("---"):
                continue

            parts = line.split()
            if len(parts) >= 3:
                name = parts[0]
                try:
                    timestamp = datetime.fromisoformat(parts[1].replace("Z", "+00:00"))
                    size = parts[2] if len(parts) > 2 else None

                    backups.append(
                        BackupInfo(
                            name=name, timestamp=timestamp, size=size, type="walg"
                        )
                    )
                except ValueError:
                    continue

        return sorted(backups, key=lambda x: x.timestamp, reverse=True)

    def restore_backup(
        self,
        executor: RemoteExecutor,
        backup_name: str = "LATEST",
        target_time: Optional[str] = None,
    ) -> bool:
        """
        Restore from WAL-G backup.

        Args:
            executor: SSH executor instance
            backup_name: Name of backup to restore (default: LATEST)
            target_time: Target time for point-in-time recovery (format: YYYY-MM-DD HH:MM:SS)

        Returns:
            True if restore was successful, False otherwise
        """
        console.print(f"[blue]Restoring from WAL-G backup: {backup_name}[/blue]")

        try:
            # Stop PostgreSQL and remove existing data
            console.print("[yellow]Stopping PostgreSQL container...[/yellow]")
            stdout, stderr, exit_code = executor.execute(
                f"docker compose -f {self.config.docker_compose_path} down --volumes"
            )

            # Remove the volume
            console.print("[yellow]Removing existing data volume...[/yellow]")
            executor.execute("docker volume rm postgres_data", show_command=False)

            # Fetch backup
            console.print(f"[yellow]Fetching backup {backup_name}...[/yellow]")
            fetch_command = (
                f"docker run --rm "
                f"--env-file {self.config.docker_compose_path.replace('docker-compose.yml', '.env')} "
                f"--user postgres "
                f"-v postgres_data:/var/lib/postgresql/data "
                f"lafayettegabe/wald:latest "
                f"envdir /etc/wal-g/env /usr/local/bin/wal-g backup-fetch /var/lib/postgresql/data {backup_name}"
            )

            stdout, stderr, exit_code = executor.execute(fetch_command)
            if exit_code != 0:
                console.print("[red]Failed to fetch backup[/red]")
                return False

            # Configure point-in-time recovery if target_time is specified
            if target_time:
                console.print(
                    f"[yellow]Configuring point-in-time recovery to {target_time}...[/yellow]"
                )
                recovery_command = (
                    f"docker run --rm "
                    f"-v postgres_data:/var/lib/postgresql/data "
                    f"lafayettegabe/wald:latest "
                    f'bash -c "'
                    f"touch /var/lib/postgresql/data/recovery.signal && "
                    f"cat >> /var/lib/postgresql/data/postgresql.auto.conf << 'EOF'\\n"
                    f"restore_command = 'envdir /etc/wal-g/env /usr/local/bin/wal-g wal-fetch %f %p'\\n"
                    f"recovery_target_time = '{target_time}'\\n"
                    f'EOF"'
                )

                stdout, stderr, exit_code = executor.execute(recovery_command)
                if exit_code != 0:
                    console.print(
                        "[red]Failed to configure point-in-time recovery[/red]"
                    )
                    return False

            # Start PostgreSQL
            console.print("[yellow]Starting PostgreSQL container...[/yellow]")
            stdout, stderr, exit_code = executor.execute(
                f"docker compose -f {self.config.docker_compose_path} up -d"
            )

            if exit_code == 0:
                console.print("[green]✓ WAL-G restore completed successfully[/green]")
                return True
            else:
                console.print("[red]✗ Failed to start PostgreSQL after restore[/red]")
                return False

        except SSHError as e:
            console.print(f"[red]✗ WAL-G restore failed: {e}[/red]")
            return False


class LogicalBackupManager:
    """Manager for logical backup operations."""

    def __init__(self, config: Config):
        self.config = config

    def create_backup(
        self, executor: RemoteExecutor, database: Optional[str] = None
    ) -> bool:
        """
        Create a logical backup.

        Args:
            executor: SSH executor instance
            database: Specific database to backup (None for all)

        Returns:
            True if backup was successful, False otherwise
        """
        if database:
            console.print(
                f"[blue]Creating logical backup for database: {database}[/blue]"
            )
            return self._create_single_database_backup(executor, database)
        else:
            console.print("[blue]Creating logical backup for all databases[/blue]")
            return self._create_all_databases_backup(executor)

    def _create_single_database_backup(
        self, executor: RemoteExecutor, database: str
    ) -> bool:
        """Create backup for a single database."""
        if not executor.check_container_running(self.config.postgres_container):
            console.print(
                f"[red]PostgreSQL container '{self.config.postgres_container}' is not running[/red]"
            )
            return False

        timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
        backup_file = f"/tmp/{database}_backup_{timestamp}.sql.gz"

        command = f"pg_dump -U postgres_{database} -d {database} | gzip > {backup_file}"

        try:
            with Progress(
                SpinnerColumn(),
                TextColumn("[progress.description]{task.description}"),
                console=console,
                transient=True,
            ) as progress:
                progress.add_task(f"Backing up {database}...", total=None)

                stdout, stderr, exit_code = executor.docker_exec(
                    self.config.postgres_container, command
                )

                if exit_code == 0:
                    console.print(
                        f"[green]✓ Logical backup created: {backup_file}[/green]"
                    )
                    return True
                else:
                    console.print(f"[red]✗ Logical backup failed for {database}[/red]")
                    if stderr:
                        console.print(f"[red]Error: {stderr}[/red]")
                    return False

        except SSHError as e:
            console.print(f"[red]✗ Logical backup failed: {e}[/red]")
            return False

    def _create_all_databases_backup(self, executor: RemoteExecutor) -> bool:
        """Create backup for all databases using db-backup container."""
        if not executor.check_container_running(self.config.backup_container):
            console.print(
                f"[red]Backup container '{self.config.backup_container}' is not running[/red]"
            )
            return False

        try:
            with Progress(
                SpinnerColumn(),
                TextColumn("[progress.description]{task.description}"),
                console=console,
                transient=True,
            ) as progress:
                progress.add_task("Creating logical backup...", total=None)

                stdout, stderr, exit_code = executor.docker_exec(
                    self.config.backup_container, "backup-now"
                )

                if exit_code == 0:
                    console.print(
                        "[green]✓ Logical backup created successfully[/green]"
                    )
                    return True
                else:
                    console.print("[red]✗ Logical backup failed[/red]")
                    if stderr:
                        console.print(f"[red]Error: {stderr}[/red]")
                    return False

        except SSHError as e:
            console.print(f"[red]✗ Logical backup failed: {e}[/red]")
            return False

    def list_backups(self, executor: RemoteExecutor) -> List[BackupInfo]:
        """
        List available logical backups.

        Args:
            executor: SSH executor instance

        Returns:
            List of BackupInfo objects
        """
        if not executor.check_container_running(self.config.backup_container):
            console.print(
                f"[red]Backup container '{self.config.backup_container}' is not running[/red]"
            )
            return []

        try:
            stdout, stderr, exit_code = executor.docker_exec(
                self.config.backup_container, "backup-list"
            )

            if exit_code != 0:
                console.print("[red]Failed to list logical backups[/red]")
                if stderr:
                    console.print(f"[red]Error: {stderr}[/red]")
                return []

            return self._parse_logical_backup_list(stdout)

        except SSHError as e:
            console.print(f"[red]Failed to list logical backups: {e}[/red]")
            return []

    def _parse_logical_backup_list(self, output: str) -> List[BackupInfo]:
        """Parse logical backup list output."""
        backups = []
        lines = output.strip().split("\n")

        for line in lines:
            line = line.strip()
            if not line:
                continue

            # This is a simplified parser - you may need to adjust based on actual output format
            if ".sql" in line or ".bz2" in line:
                try:
                    # Extract timestamp and database name from filename
                    # Format might be like: database_20240101_120000.sql.bz2
                    match = re.search(r"(\w+)_(\d{8}_\d{6})", line)
                    if match:
                        database = match.group(1)
                        timestamp_str = match.group(2)
                        timestamp = datetime.strptime(timestamp_str, "%Y%m%d_%H%M%S")

                        backups.append(
                            BackupInfo(
                                name=line,
                                timestamp=timestamp,
                                type="logical",
                                database=database,
                            )
                        )
                except ValueError:
                    continue

        return sorted(backups, key=lambda x: x.timestamp, reverse=True)

    def restore_backup(
        self, executor: RemoteExecutor, database: str, backup_file: str
    ) -> bool:
        """
        Restore from logical backup.

        Args:
            executor: SSH executor instance
            database: Target database name
            backup_file: Path to backup file

        Returns:
            True if restore was successful, False otherwise
        """
        console.print(
            f"[blue]Restoring logical backup for {database} from {backup_file}[/blue]"
        )

        if not executor.check_container_running(self.config.postgres_container):
            console.print(
                f"[red]PostgreSQL container '{self.config.postgres_container}' is not running[/red]"
            )
            return False

        try:
            # Determine restore command based on file extension
            if backup_file.endswith(".gz"):
                restore_command = (
                    f"gunzip -c {backup_file} | psql -U postgres -d {database}"
                )
            elif backup_file.endswith(".bz2"):
                restore_command = (
                    f"bunzip2 -c {backup_file} | psql -U postgres -d {database}"
                )
            else:
                restore_command = f"psql -U postgres -d {database} < {backup_file}"

            with Progress(
                SpinnerColumn(),
                TextColumn("[progress.description]{task.description}"),
                console=console,
                transient=True,
            ) as progress:
                progress.add_task(f"Restoring {database}...", total=None)

                stdout, stderr, exit_code = executor.docker_exec(
                    self.config.postgres_container, restore_command
                )

                if exit_code == 0:
                    console.print(
                        f"[green]✓ Logical restore completed for {database}[/green]"
                    )

                    # Reset database permissions
                    console.print("[yellow]Resetting database permissions...[/yellow]")
                    permission_commands = [
                        f"ALTER DATABASE {database} OWNER TO postgres_{database};",
                        f"GRANT ALL PRIVILEGES ON DATABASE {database} TO postgres_{database};",
                        f"GRANT ALL PRIVILEGES ON ALL TABLES IN SCHEMA public TO postgres_{database};",
                        f"GRANT ALL PRIVILEGES ON ALL SEQUENCES IN SCHEMA public TO postgres_{database};",
                    ]

                    for cmd in permission_commands:
                        executor.docker_exec(
                            self.config.postgres_container,
                            f'psql -U postgres -c "{cmd}" {database}',
                        )

                    return True
                else:
                    console.print(f"[red]✗ Logical restore failed for {database}[/red]")
                    if stderr:
                        console.print(f"[red]Error: {stderr}[/red]")
                    return False

        except SSHError as e:
            console.print(f"[red]✗ Logical restore failed: {e}[/red]")
            return False



"""SSH remote execution utilities for DuplicAid."""

from typing import Optional, Tuple

import paramiko
from rich.console import Console
from rich.live import Live
from rich.spinner import Spinner

from .config import Config
from .executor import BaseExecutor, ExecutorError

console = Console()


class SSHError(ExecutorError):
    pass


class RemoteExecutor(BaseExecutor):
    """SSH client wrapper for executing commands on remote server."""

    def __init__(self, config: Config):
        self.config = config
        self.client: Optional[paramiko.SSHClient] = None

    def __enter__(self):
        """Context manager entry."""
        self.connect()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """Context manager exit."""
        self.disconnect()

    def connect(self) -> None:
        """Establish SSH connection to remote server."""
        if not self.config.validate():
            raise SSHError("Invalid configuration")

        try:
            self.client = paramiko.SSHClient()
            self.client.set_missing_host_key_policy(paramiko.AutoAddPolicy())

            with Live(
                Spinner("dots", text=f"Connecting to {self.config.remote_host}..."),
                console=console,
                transient=True,
            ):
                self.client.connect(
                    hostname=self.config.remote_host,
                    port=self.config.remote_port,
                    username=self.config.remote_user,
                    key_filename=self.config.ssh_key_path,
                    timeout=10,
                )

            console.print(f"[green]✓ Connected to {self.config.remote_host}[/green]")

        except Exception as e:
            raise SSHError(f"Failed to connect to {self.config.remote_host}: {e}")

    def disconnect(self) -> None:
        """Close SSH connection."""
        if self.client:
            self.client.close()
            self.client = None

    def execute(self, command: str, show_command: bool = True) -> Tuple[str, str, int]:
        """
        Execute command on remote server.

        Args:
            command: Command to execute
            show_command: Whether to display the command being executed

        Returns:
            Tuple of (stdout, stderr, exit_code)
        """
        if not self.client:
            raise SSHError("Not connected to remote server")

        if show_command:
            console.print(f"[dim]$ {command}[/dim]")

        try:
            stdin, stdout, stderr = self.client.exec_command(command)
            exit_code = stdout.channel.recv_exit_status()

            stdout_text = stdout.read().decode("utf-8").strip()
            stderr_text = stderr.read().decode("utf-8").strip()

            if exit_code != 0:
                console.print(f"[red]Command failed with exit code {exit_code}[/red]")
                if stderr_text:
                    console.print(f"[red]Error: {stderr_text}[/red]")

            return stdout_text, stderr_text, exit_code

        except Exception as e:
            raise SSHError(f"Failed to execute command: {e}")

    def docker_exec(
        self, container: str, command: str, user: Optional[str] = None
    ) -> Tuple[str, str, int]:
        """
        Execute command in Docker container.

        Args:
            container: Container name
            command: Command to execute in container
            user: User to run command as

        Returns:
            Tuple of (stdout, stderr, exit_code)
        """
        docker_command = "docker exec"
        if user:
            docker_command += f" -u {user}"
        docker_command += f" {container} {command}"

        return self.execute(docker_command)

    def docker_exec_interactive(
        self, container: str, command: str, stdin_data: str = None
    ) -> Tuple[str, str, int]:
        """
        Execute interactive command in Docker container (e.g., for piping data).

        Args:
            container: Container name
            command: Command to execute in container
            stdin_data: Data to pipe to stdin

        Returns:
            Tuple of (stdout, stderr, exit_code)
        """
        docker_command = f"docker exec -i {container} {command}"

        if stdin_data:
            # For commands that need stdin, we'll use a different approach
            full_command = f"echo '{stdin_data}' | {docker_command}"
            return self.execute(full_command)
        else:
            return self.execute(docker_command)

    def check_container_running(self, container: str) -> bool:
        """
        Check if a Docker container is running.

        Args:
            container: Container name

        Returns:
            True if container is running, False otherwise
        """
        stdout, stderr, exit_code = self.execute(
            f"docker ps --filter name={container} --filter status=running --format '{{{{.Names}}}}'",
            show_command=False,
        )

        return container in stdout.split("\n")

    def get_container_status(self, container: str) -> Optional[str]:
        """
        Get container status.

        Args:
            container: Container name

        Returns:
            Container status or None if not found
        """
        stdout, stderr, exit_code = self.execute(
            f"docker ps -a --filter name={container} --format '{{{{.Status}}}}'",
            show_command=False,
        )

        if stdout:
            return stdout.strip()
        return None

    def file_exists(self, path: str) -> bool:
        """
        Check if file exists on remote server.

        Args:
            path: File path to check

        Returns:
            True if file exists, False otherwise
        """
        stdout, stderr, exit_code = self.execute(f"test -f {path}", show_command=False)
        return exit_code == 0



from unittest.mock import Mock

import pytest

from duplicaid.backup import LogicalBackupManager, WALGBackupManager
from duplicaid.config import Config


@pytest.fixture
def test_config():
    config = Config()
    config._data = {
        "execution_mode": "local",
        "containers": {"postgres": "postgres", "backup": "db-backup"},
        "paths": {"docker_compose": "/test/docker-compose.yml"},
        "databases": ["testdb1", "testdb2"],
    }
    return config


@pytest.fixture
def mock_executor():
    executor = Mock()
    executor.docker_exec.return_value = ("", "", 0)
    executor.check_container_running.return_value = True
    return executor


def test_walg_backup_creation(test_config, mock_executor):
    manager = WALGBackupManager(test_config)
    mock_executor.docker_exec.return_value = ("backup_20240101_120000", "", 0)

    manager.create_backup(mock_executor)
    mock_executor.docker_exec.assert_called()


def test_logical_backup_all_databases(test_config, mock_executor):
    manager = LogicalBackupManager(test_config)

    manager.create_backup(mock_executor)
    mock_executor.docker_exec.assert_called()


def test_logical_backup_specific_database(test_config, mock_executor):
    manager = LogicalBackupManager(test_config)

    manager.create_backup(mock_executor, "testdb1")
    mock_executor.docker_exec.assert_called()


def test_walg_backup_list(test_config, mock_executor):
    manager = WALGBackupManager(test_config)
    mock_executor.docker_exec.return_value = ("backup1\nbackup2", "", 0)

    manager.list_backups(mock_executor)
    mock_executor.docker_exec.assert_called()


def test_logical_backup_list(test_config, mock_executor):
    manager = LogicalBackupManager(test_config)
    mock_executor.docker_exec.return_value = ("file1.sql.gz\nfile2.sql.gz", "", 0)

    manager.list_backups(mock_executor)
    mock_executor.docker_exec.assert_called()


def test_walg_restore_latest(test_config, mock_executor):
    manager = WALGBackupManager(test_config)
    mock_executor.execute.return_value = ("", "", 0)

    manager.restore_backup(mock_executor)
    mock_executor.execute.assert_called()


def test_walg_restore_specific_backup(test_config, mock_executor):
    manager = WALGBackupManager(test_config)
    mock_executor.execute.return_value = ("", "", 0)

    manager.restore_backup(mock_executor, backup_name="backup_20240101")
    mock_executor.execute.assert_called()


def test_logical_restore(test_config, mock_executor):
    manager = LogicalBackupManager(test_config)

    manager.restore_backup(mock_executor, "testdb1", "/path/to/backup.sql.gz")
    mock_executor.docker_exec.assert_called()


def test_backup_manager_initialization(test_config):
    walg_manager = WALGBackupManager(test_config)
    logical_manager = LogicalBackupManager(test_config)

    assert walg_manager.config == test_config
    assert logical_manager.config == test_config


def test_container_status_check(test_config, mock_executor):
    manager = WALGBackupManager(test_config)
    mock_executor.check_container_running.return_value = False

    manager.create_backup(mock_executor)
    mock_executor.check_container_running.assert_called()



"""Database discovery functionality for DuplicAid."""

from typing import Any, Dict, List

from rich.console import Console

from .config import Config
from .ssh import RemoteExecutor, SSHError

console = Console()


class DatabaseDiscovery:
    """Database discovery and information gathering."""

    def __init__(self, config: Config):
        self.config = config

    def get_databases(self, executor: RemoteExecutor) -> List[Dict[str, Any]]:
        """
        Discover databases on the remote PostgreSQL server.

        Args:
            executor: SSH executor instance

        Returns:
            List of database information dictionaries
        """
        if not executor.check_container_running(self.config.postgres_container):
            console.print(
                f"[red]PostgreSQL container '{self.config.postgres_container}' is not running[/red]"
            )
            return []

        try:
            # Query for database names and sizes
            query = """
            SELECT datname, pg_size_pretty(pg_database_size(datname)) as size
            FROM pg_database
            WHERE datistemplate = false
            AND datname != 'postgres'
            ORDER BY datname;
            """

            command = f'psql -U postgres -t -c "{query}"'

            stdout, stderr, exit_code = executor.docker_exec(
                self.config.postgres_container, command
            )

            if exit_code != 0:
                console.print("[red]Failed to query database information[/red]")
                if stderr:
                    console.print(f"[red]Error: {stderr}[/red]")
                return []

            return self._parse_database_list(stdout)

        except SSHError as e:
            console.print(f"[red]Failed to discover databases: {e}[/red]")
            return []

    def _parse_database_list(self, output: str) -> List[Dict[str, Any]]:
        """Parse database list output from PostgreSQL."""
        databases = []
        lines = output.strip().split("\n")

        for line in lines:
            line = line.strip()
            if not line or line.startswith("datname"):
                continue

            # Split by pipe character (PostgreSQL default separator)
            parts = [part.strip() for part in line.split("|")]
            if len(parts) >= 2:
                name = parts[0]
                size = parts[1]

                # Skip template and system databases
                if name not in ["template0", "template1", "postgres"]:
                    databases.append({"name": name, "size": size})

        return databases

    def get_database_info(
        self, executor: RemoteExecutor, database: str
    ) -> Dict[str, Any]:
        """
        Get detailed information about a specific database.

        Args:
            executor: SSH executor instance
            database: Database name

        Returns:
            Dictionary with database information
        """
        if not executor.check_container_running(self.config.postgres_container):
            console.print(
                f"[red]PostgreSQL container '{self.config.postgres_container}' is not running[/red]"
            )
            return {}

        try:
            # Query for detailed database information
            query = f"""
            SELECT
                datname,
                pg_size_pretty(pg_database_size(datname)) as size,
                datcollate,
                datctype,
                datistemplate,
                datallowconn,
                datconnlimit
            FROM pg_database
            WHERE datname = '{database}';
            """

            command = f'psql -U postgres -t -c "{query}"'

            stdout, stderr, exit_code = executor.docker_exec(
                self.config.postgres_container, command
            )

            if exit_code != 0:
                console.print(
                    f"[red]Failed to query information for database '{database}'[/red]"
                )
                return {}

            return self._parse_database_info(stdout, database)

        except SSHError as e:
            console.print(f"[red]Failed to get database info: {e}[/red]")
            return {}

    def _parse_database_info(self, output: str, database: str) -> Dict[str, Any]:
        """Parse detailed database information."""
        lines = output.strip().split("\n")

        for line in lines:
            line = line.strip()
            if not line:
                continue

            parts = [part.strip() for part in line.split("|")]
            if len(parts) >= 7 and parts[0] == database:
                return {
                    "name": parts[0],
                    "size": parts[1],
                    "collate": parts[2],
                    "ctype": parts[3],
                    "is_template": parts[4] == "t",
                    "allow_conn": parts[5] == "t",
                    "conn_limit": parts[6],
                }

        return {}

    def check_database_exists(self, executor: RemoteExecutor, database: str) -> bool:
        """
        Check if a database exists.

        Args:
            executor: SSH executor instance
            database: Database name to check

        Returns:
            True if database exists, False otherwise
        """
        databases = self.get_databases(executor)
        return any(db["name"] == database for db in databases)



import subprocess
import time
from pathlib import Path

import pytest

from duplicaid.config import Config
from duplicaid.local import LocalExecutor


@pytest.fixture(scope="session")
def docker_compose_file():
    return Path(__file__).parent.parent / "docker-compose.test.yml"


@pytest.fixture(scope="session")
def test_services(docker_compose_file):
    subprocess.run(
        ["docker", "compose", "-f", str(docker_compose_file), "down", "-v"],
        capture_output=True,
    )

    result = subprocess.run(
        ["docker", "compose", "-f", str(docker_compose_file), "up", "-d"],
        capture_output=True,
        text=True,
    )

    if result.returncode != 0:
        pytest.fail(f"Failed to start test services: {result.stderr}")

    time.sleep(30)

    yield

    subprocess.run(
        ["docker", "compose", "-f", str(docker_compose_file), "down", "-v"],
        capture_output=True,
    )


@pytest.fixture
def test_config(docker_compose_file):
    config = Config()
    config._data = {
        "execution_mode": "local",
        "containers": {"postgres": "postgres", "backup": "db-backup"},
        "paths": {"docker_compose": str(docker_compose_file)},
        "databases": ["testdb1", "testdb2"],
    }
    return config


@pytest.fixture
def local_executor(test_config):
    return LocalExecutor(test_config)


@pytest.fixture
def postgres_ready(test_services, local_executor):
    max_retries = 30
    for i in range(max_retries):
        if local_executor.check_container_running("postgres"):
            stdout, stderr, exit_code = local_executor.docker_exec(
                "postgres", "pg_isready -U postgres -d postgres"
            )
            if exit_code == 0:
                break
        time.sleep(1)
    else:
        pytest.fail("PostgreSQL not ready after 30 seconds")

    stdout, stderr, exit_code = local_executor.docker_exec(
        "postgres", 'psql -U postgres -c "CREATE DATABASE testdb1;" || true'
    )
    stdout, stderr, exit_code = local_executor.docker_exec(
        "postgres", 'psql -U postgres -c "CREATE DATABASE testdb2;" || true'
    )



"""Configuration management for DuplicAid."""

import os
from pathlib import Path
from typing import List, Optional

import yaml
from rich.console import Console
from rich.prompt import Prompt

console = Console()


class Config:
    """Configuration manager for DuplicAid."""

    def __init__(self, config_path: Optional[Path] = None):
        self.config_path = config_path or Path.home() / ".duplicaid" / "config.yml"
        self._data = {}
        self.load()

    def load(self) -> None:
        """Load configuration from file."""
        if self.config_path.exists():
            try:
                with open(self.config_path, "r") as f:
                    self._data = yaml.safe_load(f) or {}
            except Exception as e:
                console.print(f"[red]Error loading config: {e}[/red]")
                self._data = {}
        else:
            self._data = {}

    def save(self) -> None:
        """Save configuration to file."""
        self.config_path.parent.mkdir(parents=True, exist_ok=True)
        try:
            with open(self.config_path, "w") as f:
                yaml.dump(self._data, f, default_flow_style=False, indent=2)
            console.print(f"[green]Configuration saved to {self.config_path}[/green]")
        except Exception as e:
            console.print(f"[red]Error saving config: {e}[/red]")

    @property
    def execution_mode(self) -> str:
        """Get execution mode (remote or local)."""
        return self._data.get("execution_mode", "remote")

    @property
    def remote_host(self) -> Optional[str]:
        """Get remote host address."""
        return self._data.get("remote", {}).get("host")

    @property
    def remote_user(self) -> Optional[str]:
        """Get remote SSH user."""
        return self._data.get("remote", {}).get("user", "root")

    @property
    def ssh_key_path(self) -> Optional[str]:
        """Get SSH private key path."""
        return self._data.get("remote", {}).get("ssh_key_path")

    @property
    def remote_port(self) -> int:
        """Get remote SSH port."""
        return self._data.get("remote", {}).get("port", 22)

    @property
    def postgres_container(self) -> str:
        """Get PostgreSQL container name."""
        return self._data.get("containers", {}).get("postgres", "postgres")

    @property
    def backup_container(self) -> str:
        """Get backup container name."""
        return self._data.get("containers", {}).get("backup", "db-backup")

    @property
    def docker_compose_path(self) -> str:
        """Get Docker Compose file path on remote server."""
        return self._data.get("paths", {}).get(
            "docker_compose", "/home/correlaid/postgres/docker-compose.yml"
        )

    @property
    def databases(self) -> List[str]:
        """Get list of configured databases."""
        return self._data.get("databases", [])

    def init_config(self) -> None:
        """Initialize configuration interactively."""
        console.print("[bold blue]DuplicAid Configuration Setup[/bold blue]")
        console.print("Please provide the following information:\n")

        # Execution mode
        mode = Prompt.ask(
            "Execution mode", choices=["remote", "local"], default="remote"
        )

        # Remote server configuration (only if remote mode)
        if mode == "remote":
            host = Prompt.ask("Remote server hostname/IP")
            user = Prompt.ask("SSH username", default="root")
            port = Prompt.ask("SSH port", default="22")

            ssh_key_default = str(Path.home() / ".ssh" / "id_rsa")
            ssh_key = Prompt.ask("SSH private key path", default=ssh_key_default)
        else:
            host = user = port = ssh_key = None

        # Container configuration
        postgres_container = Prompt.ask("PostgreSQL container name", default="postgres")
        backup_container = Prompt.ask("Backup container name", default="db-backup")

        # Paths
        if mode == "remote":
            compose_path = Prompt.ask(
                "Docker Compose file path on remote",
                default="/home/correlaid/postgres/docker-compose.yml",
            )
        else:
            compose_path = Prompt.ask(
                "Docker Compose file path",
                default="./docker-compose.test.yml",
            )

        # Databases
        console.print("\n[yellow]Database configuration:[/yellow]")
        databases = []
        while True:
            db = Prompt.ask("Database name (press Enter to finish)", default="")
            if not db:
                break
            databases.append(db)

        if not databases:
            console.print(
                "[yellow]No databases configured. You can add them later with "
                "'duplicaid config add-db'[/yellow]"
            )

        # Save configuration
        self._data = {
            "execution_mode": mode,
            "containers": {"postgres": postgres_container, "backup": backup_container},
            "paths": {"docker_compose": compose_path},
            "databases": databases,
        }

        if mode == "remote":
            self._data["remote"] = {
                "host": host,
                "user": user,
                "port": int(port),
                "ssh_key_path": ssh_key,
            }

        self.save()
        console.print("\n[green]✓ Configuration completed![/green]")

    def add_database(self, database: str) -> None:
        """Add a database to the configuration."""
        if "databases" not in self._data:
            self._data["databases"] = []

        if database not in self._data["databases"]:
            self._data["databases"].append(database)
            self.save()
            console.print(f"[green]✓ Added database: {database}[/green]")
        else:
            console.print(f"[yellow]Database {database} already configured[/yellow]")

    def remove_database(self, database: str) -> None:
        """Remove a database from the configuration."""
        if "databases" in self._data and database in self._data["databases"]:
            self._data["databases"].remove(database)
            self.save()
            console.print(f"[green]✓ Removed database: {database}[/green]")
        else:
            console.print(
                f"[yellow]Database {database} not found in configuration[/yellow]"
            )

    def is_configured(self) -> bool:
        """Check if configuration is complete."""
        if self.execution_mode == "local":
            return bool(self.postgres_container and self.docker_compose_path)
        else:
            return bool(
                self.remote_host
                and self.ssh_key_path
                and os.path.exists(self.ssh_key_path)
            )

    def validate(self) -> bool:
        """Validate configuration."""
        if self.execution_mode == "local":
            if not self.docker_compose_path:
                console.print("[red]Missing Docker Compose path configuration[/red]")
                return False
            return True
        else:
            if not self.remote_host:
                console.print("[red]Missing remote host configuration[/red]")
                return False

            if not self.ssh_key_path:
                console.print("[red]Missing SSH key path configuration[/red]")
                return False

            if not os.path.exists(self.ssh_key_path):
                console.print(f"[red]SSH key not found: {self.ssh_key_path}[/red]")
                return False

            return True



from duplicaid.config import Config


def test_config_default_values():
    config = Config()
    assert config.execution_mode == "remote"
    assert config.postgres_container == "postgres"
    assert config.backup_container == "db-backup"


def test_config_local_mode():
    config = Config()
    config._data = {
        "execution_mode": "local",
        "containers": {"postgres": "test-postgres", "backup": "test-backup"},
        "paths": {"docker_compose": "./test.yml"},
        "databases": ["testdb"],
    }

    assert config.execution_mode == "local"
    assert config.postgres_container == "test-postgres"
    assert config.backup_container == "test-backup"
    assert config.docker_compose_path == "./test.yml"
    assert config.databases == ["testdb"]


def test_config_remote_mode():
    config = Config()
    config._data = {
        "execution_mode": "remote",
        "remote": {
            "host": "test.example.com",
            "user": "testuser",
            "port": 2222,
            "ssh_key_path": "/test/key",
        },
        "containers": {"postgres": "postgres", "backup": "db-backup"},
        "paths": {"docker_compose": "/remote/compose.yml"},
        "databases": ["db1", "db2"],
    }

    assert config.execution_mode == "remote"
    assert config.remote_host == "test.example.com"
    assert config.remote_user == "testuser"
    assert config.remote_port == 2222
    assert config.ssh_key_path == "/test/key"


def test_config_validation_local():
    config = Config()
    config._data = {
        "execution_mode": "local",
        "containers": {"postgres": "postgres", "backup": "db-backup"},
        "paths": {"docker_compose": "./test.yml"},
    }

    assert config.validate()


def test_config_validation_remote_missing_host():
    config = Config()
    config._data = {
        "execution_mode": "remote",
        "containers": {"postgres": "postgres", "backup": "db-backup"},
    }

    assert not config.validate()


def test_add_remove_database():
    config = Config()
    config._data = {"databases": ["existing_db"]}

    config.add_database("new_db")
    assert "new_db" in config.databases

    config.remove_database("new_db")
    assert "new_db" not in config.databases



from abc import ABC, abstractmethod
from typing import Optional, Tuple


class ExecutorError(Exception):
    pass


class BaseExecutor(ABC):
    @abstractmethod
    def __enter__(self):
        pass

    @abstractmethod
    def __exit__(self, exc_type, exc_val, exc_tb):
        pass

    @abstractmethod
    def execute(self, command: str, show_command: bool = True) -> Tuple[str, str, int]:
        pass

    @abstractmethod
    def docker_exec(
        self, container: str, command: str, user: Optional[str] = None
    ) -> Tuple[str, str, int]:
        pass

    @abstractmethod
    def docker_exec_interactive(
        self, container: str, command: str, stdin_data: str = None
    ) -> Tuple[str, str, int]:
        pass

    @abstractmethod
    def check_container_running(self, container: str) -> bool:
        pass

    @abstractmethod
    def get_container_status(self, container: str) -> Optional[str]:
        pass

    @abstractmethod
    def file_exists(self, path: str) -> bool:
        pass



import pytest

from duplicaid.backup import LogicalBackupManager, WALGBackupManager
from duplicaid.discovery import DatabaseDiscovery


@pytest.mark.integration
def test_database_discovery(test_services, local_executor, postgres_ready):
    discovery = DatabaseDiscovery(local_executor.config)
    databases = discovery.get_databases(local_executor)

    db_names = [db["name"] for db in databases]
    assert "testdb1" in db_names or "testdb2" in db_names


@pytest.mark.integration
def test_container_status_check(test_services, local_executor):
    assert local_executor.check_container_running("postgres")
    assert not local_executor.check_container_running("nonexistent")

    status = local_executor.get_container_status("postgres")
    assert status is not None
    assert "Up" in status


@pytest.mark.integration
def test_walg_backup_manager_init(test_services, local_executor, postgres_ready):
    walg_manager = WALGBackupManager(local_executor.config)
    assert walg_manager.config == local_executor.config


@pytest.mark.integration
def test_logical_backup_manager_init(test_services, local_executor, postgres_ready):
    logical_manager = LogicalBackupManager(local_executor.config)
    assert logical_manager.config == local_executor.config


@pytest.mark.integration
def test_postgres_connectivity(test_services, local_executor, postgres_ready):
    stdout, stderr, exit_code = local_executor.docker_exec(
        "postgres", "psql -U postgres -d postgres -c 'SELECT 1;'"
    )
    assert exit_code == 0


@pytest.mark.integration
def test_database_creation(test_services, local_executor, postgres_ready):
    stdout, stderr, exit_code = local_executor.docker_exec(
        "postgres", "psql -U postgres -c 'CREATE DATABASE test_integration;'"
    )
    assert exit_code == 0

    stdout, stderr, exit_code = local_executor.docker_exec(
        "postgres", "psql -U postgres -c 'DROP DATABASE test_integration;'"
    )
    assert exit_code == 0



"""Local execution utilities for DuplicAid."""

import os
import subprocess
from typing import Optional, Tuple

from rich.console import Console

from .config import Config
from .executor import BaseExecutor, ExecutorError

console = Console()


class LocalError(ExecutorError):
    pass


class LocalExecutor(BaseExecutor):
    def __init__(self, config: Config):
        self.config = config

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        pass

    def execute(self, command: str, show_command: bool = True) -> Tuple[str, str, int]:
        if show_command:
            console.print(f"[dim]$ {command}[/dim]")

        try:
            result = subprocess.run(
                command, shell=True, capture_output=True, text=True, timeout=300
            )

            if result.returncode != 0:
                console.print(
                    f"[red]Command failed with exit code {result.returncode}[/red]"
                )
                if result.stderr:
                    console.print(f"[red]Error: {result.stderr.strip()}[/red]")

            return result.stdout.strip(), result.stderr.strip(), result.returncode

        except subprocess.TimeoutExpired:
            raise LocalError("Command timed out")
        except Exception as e:
            raise LocalError(f"Failed to execute command: {e}")

    def docker_exec(
        self, container: str, command: str, user: Optional[str] = None
    ) -> Tuple[str, str, int]:
        docker_command = "docker exec"
        if user:
            docker_command += f" -u {user}"
        docker_command += f" {container} {command}"

        return self.execute(docker_command)

    def docker_exec_interactive(
        self, container: str, command: str, stdin_data: str = None
    ) -> Tuple[str, str, int]:
        docker_command = f"docker exec -i {container} {command}"

        if stdin_data:
            full_command = f"echo '{stdin_data}' | {docker_command}"
            return self.execute(full_command)
        else:
            return self.execute(docker_command)

    def check_container_running(self, container: str) -> bool:
        stdout, stderr, exit_code = self.execute(
            f"docker ps --filter name={container} --filter status=running --format '{{{{.Names}}}}'",
            show_command=False,
        )

        return container in stdout.split("\n")

    def get_container_status(self, container: str) -> Optional[str]:
        stdout, stderr, exit_code = self.execute(
            f"docker ps -a --filter name={container} --format '{{{{.Status}}}}'",
            show_command=False,
        )

        if stdout:
            return stdout.strip()
        return None

    def file_exists(self, path: str) -> bool:
        return os.path.exists(path)



from typer.testing import CliRunner

from duplicaid.cli import app

runner = CliRunner()


def test_cli_help():
    result = runner.invoke(app, ["--help"])
    assert result.exit_code == 0
    assert "PostgreSQL backup management CLI tool" in result.stdout


def test_config_commands():
    result = runner.invoke(app, ["config", "--help"])
    assert result.exit_code == 0
    assert "Configuration management" in result.stdout


def test_backup_commands():
    result = runner.invoke(app, ["backup", "--help"])
    assert result.exit_code == 0
    assert "Create backups" in result.stdout


def test_status_without_config():
    result = runner.invoke(app, ["status"])
    assert result.exit_code == 1
    assert "Configuration not found" in result.stdout


def test_config_show_empty():
    result = runner.invoke(app, ["config", "show"])
    assert (
        "No configuration found" in result.stdout
        or "DuplicAid Configuration" in result.stdout
    )


def test_backup_walg_help():
    result = runner.invoke(app, ["backup", "walg", "--help"])
    assert result.exit_code == 0
    assert "Create a WAL-G backup" in result.stdout


def test_backup_logical_help():
    result = runner.invoke(app, ["backup", "logical", "--help"])
    assert result.exit_code == 0
    assert "Create a logical backup" in result.stdout


def test_list_walg_help():
    result = runner.invoke(app, ["list", "walg", "--help"])
    assert result.exit_code == 0
    assert "List available WAL-G backups" in result.stdout


def test_status_help():
    result = runner.invoke(app, ["status", "--help"])
    assert result.exit_code == 0
    assert "Show system status" in result.stdout


def test_restore_commands():
    result = runner.invoke(app, ["restore", "--help"])
    assert result.exit_code == 0
    assert "Restore from backups" in result.stdout



from unittest.mock import Mock, patch

import pytest

from duplicaid.config import Config
from duplicaid.ssh import RemoteExecutor, SSHError


@pytest.fixture
def remote_config():
    config = Config()
    config._data = {
        "execution_mode": "remote",
        "remote": {
            "host": "test.example.com",
            "user": "testuser",
            "port": 22,
            "ssh_key_path": "/path/to/key",
        },
        "containers": {"postgres": "postgres", "backup": "db-backup"},
        "paths": {"docker_compose": "/test/docker-compose.yml"},
        "databases": ["testdb"],
    }
    yield config


@pytest.fixture
def mock_ssh_client():
    with patch("paramiko.SSHClient") as mock:
        yield mock


@patch("os.path.exists", return_value=True)
def test_remote_executor_connect(mock_exists, mock_ssh_client, remote_config):
    executor = RemoteExecutor(remote_config)
    with executor:
        mock_ssh_client.return_value.connect.assert_called_once()


@patch("os.path.exists", return_value=True)
def test_remote_executor_command_execution(mock_exists, mock_ssh_client, remote_config):
    mock_stdout = Mock()
    mock_stdout.read.return_value = b"test output"
    mock_stderr = Mock()
    mock_stderr.read.return_value = b""
    mock_ssh_client.return_value.exec_command.return_value = (
        None,
        mock_stdout,
        mock_stderr,
    )

    executor = RemoteExecutor(remote_config)
    with executor:
        stdout, stderr, code = executor.execute("test command")
        assert stdout == "test output"
        mock_ssh_client.return_value.exec_command.assert_called_with("test command")


def test_remote_executor_connection_error(mock_ssh_client, remote_config):
    mock_ssh_client.return_value.connect.side_effect = Exception("Connection failed")

    executor = RemoteExecutor(remote_config)
    with pytest.raises(SSHError):
        executor.connect()


@patch("os.path.exists", return_value=True)
def test_remote_executor_file_exists(mock_exists, mock_ssh_client, remote_config):
    executor = RemoteExecutor(remote_config)

    # Mock the execute method directly to return success
    with patch.object(executor, "execute", return_value=("", "", 0)):
        with executor:
            assert executor.file_exists("/test/path")


@patch("os.path.exists", return_value=True)
def test_remote_executor_file_not_exists(mock_exists, mock_ssh_client, remote_config):
    executor = RemoteExecutor(remote_config)

    # Mock the execute method directly to return failure
    with patch.object(executor, "execute", return_value=("", "", 1)):
        with executor:
            assert not executor.file_exists("/nonexistent/path")



import pytest

from duplicaid.config import Config
from duplicaid.local import LocalExecutor


@pytest.fixture
def local_config():
    config = Config()
    config._data = {
        "execution_mode": "local",
        "containers": {"postgres": "postgres", "backup": "db-backup"},
        "paths": {"docker_compose": "./test.yml"},
    }
    return config


def test_local_executor_basic_commands(local_config):
    executor = LocalExecutor(local_config)

    with executor:
        stdout, stderr, exit_code = executor.execute("echo hello", show_command=False)
        assert stdout == "hello"
        assert exit_code == 0


def test_local_executor_failed_command(local_config):
    executor = LocalExecutor(local_config)

    with executor:
        stdout, stderr, exit_code = executor.execute("false", show_command=False)
        assert exit_code == 1


def test_local_executor_file_exists(local_config):
    executor = LocalExecutor(local_config)

    with executor:
        assert executor.file_exists("/etc/passwd")
        assert not executor.file_exists("/nonexistent/file")


def test_local_executor_docker_commands(local_config):
    executor = LocalExecutor(local_config)

    with executor:
        stdout, stderr, exit_code = executor.execute(
            "docker --version", show_command=False
        )
        assert "Docker version" in stdout or exit_code != 0



"""CLI interface for DuplicAid."""

from typing import Optional

import typer
from rich import box
from rich.console import Console
from rich.table import Table

from .backup import LogicalBackupManager, WALGBackupManager
from .config import Config
from .discovery import DatabaseDiscovery
from .executor import ExecutorError
from .local import LocalExecutor
from .ssh import RemoteExecutor

console = Console()
app = typer.Typer(
    name="duplicaid",
    help="PostgreSQL backup management CLI tool",
    rich_markup_mode="rich",
)

# Subcommands
config_app = typer.Typer(name="config", help="Configuration management")
backup_app = typer.Typer(name="backup", help="Create backups")
restore_app = typer.Typer(name="restore", help="Restore from backups")
list_app = typer.Typer(name="list", help="List available backups")

app.add_typer(config_app)
app.add_typer(backup_app)
app.add_typer(restore_app)
app.add_typer(list_app)

# Global configuration instance
config = Config()


def get_executor():
    """Get the appropriate executor based on configuration."""
    if config.execution_mode == "local":
        return LocalExecutor(config)
    else:
        return RemoteExecutor(config)


def check_config() -> bool:
    """Check if configuration is valid."""
    if not config.is_configured():
        console.print("[red]Configuration not found or incomplete.[/red]")
        console.print("Run [bold]duplicaid config init[/bold] to set up configuration.")
        raise typer.Exit(1)

    if not config.validate():
        console.print("[red]Configuration validation failed.[/red]")
        raise typer.Exit(1)

    return True


# Configuration commands
@config_app.command("init")
def config_init():
    """Initialize DuplicAid configuration."""
    config.init_config()


@config_app.command("show")
def config_show():
    """Show current configuration."""
    if not config.is_configured():
        console.print(
            "[yellow]No configuration found. Run 'duplicaid config init' to set up.[/yellow]"
        )
        return

    table = Table(title="DuplicAid Configuration", box=box.ROUNDED)
    table.add_column("Setting", style="cyan")
    table.add_column("Value", style="green")

    table.add_row("Execution Mode", config.execution_mode)

    if config.execution_mode == "remote":
        table.add_row("Remote Host", config.remote_host or "Not set")
        table.add_row("Remote User", config.remote_user or "Not set")
        table.add_row("Remote Port", str(config.remote_port))
        table.add_row("SSH Key Path", config.ssh_key_path or "Not set")

    table.add_row("PostgreSQL Container", config.postgres_container)
    table.add_row("Backup Container", config.backup_container)
    table.add_row("Docker Compose Path", config.docker_compose_path)
    table.add_row(
        "Databases", ", ".join(config.databases) if config.databases else "None"
    )

    console.print(table)


# Backup commands
@backup_app.command("walg")
def backup_walg():
    """Create a WAL-G backup (point-in-time)."""
    check_config()

    try:
        with get_executor() as executor:
            walg_manager = WALGBackupManager(config)
            success = walg_manager.create_backup(executor)

            if not success:
                raise typer.Exit(1)

    except ExecutorError as e:
        console.print(f"[red]Executor Error: {e}[/red]")
        raise typer.Exit(1)


@backup_app.command("logical")
def backup_logical(
    database: Optional[str] = typer.Option(
        None, "--db", help="Specific database to backup"
    ),
):
    """Create a logical backup."""
    check_config()

    if database and database not in config.databases:
        console.print(f"[red]Database '{database}' not found in configuration.[/red]")
        console.print("Available databases:", ", ".join(config.databases))
        raise typer.Exit(1)

    try:
        with get_executor() as executor:
            logical_manager = LogicalBackupManager(config)
            success = logical_manager.create_backup(executor, database)

            if not success:
                raise typer.Exit(1)

    except ExecutorError as e:
        console.print(f"[red]Executor Error: {e}[/red]")
        raise typer.Exit(1)


# Restore commands
@restore_app.command("walg")
def restore_walg(
    backup_name: str = typer.Option(
        "LATEST", "--backup", help="Backup name to restore"
    ),
    target_time: Optional[str] = typer.Option(
        None,
        "--to",
        help="Target time for point-in-time recovery (YYYY-MM-DD HH:MM:SS)",
    ),
):
    """Restore from WAL-G backup."""
    check_config()

    # Confirm destructive operation
    if not typer.confirm("This will destroy current data. Are you sure?"):
        console.print("[yellow]Operation cancelled.[/yellow]")
        raise typer.Exit()

    try:
        with get_executor() as executor:
            walg_manager = WALGBackupManager(config)
            success = walg_manager.restore_backup(executor, backup_name, target_time)

            if not success:
                raise typer.Exit(1)

    except ExecutorError as e:
        console.print(f"[red]Executor Error: {e}[/red]")
        raise typer.Exit(1)


@restore_app.command("logical")
def restore_logical(
    database: str = typer.Argument(..., help="Database name to restore"),
    backup_file: str = typer.Argument(..., help="Path to backup file on remote server"),
):
    """Restore from logical backup."""
    check_config()

    if database not in config.databases:
        console.print(f"[red]Database '{database}' not found in configuration.[/red]")
        console.print("Available databases:", ", ".join(config.databases))
        raise typer.Exit(1)

    # Confirm destructive operation
    if not typer.confirm(f"This will overwrite database '{database}'. Are you sure?"):
        console.print("[yellow]Operation cancelled.[/yellow]")
        raise typer.Exit()

    try:
        with get_executor() as executor:
            logical_manager = LogicalBackupManager(config)
            success = logical_manager.restore_backup(executor, database, backup_file)

            if not success:
                raise typer.Exit(1)

    except ExecutorError as e:
        console.print(f"[red]Executor Error: {e}[/red]")
        raise typer.Exit(1)


# List commands
@list_app.command("walg")
def list_walg():
    """List available WAL-G backups."""
    check_config()

    try:
        with get_executor() as executor:
            walg_manager = WALGBackupManager(config)
            backups = walg_manager.list_backups(executor)

            if not backups:
                console.print("[yellow]No WAL-G backups found.[/yellow]")
                return

            table = Table(title="WAL-G Backups", box=box.ROUNDED)
            table.add_column("Name", style="cyan")
            table.add_column("Timestamp", style="green")
            table.add_column("Size", style="yellow")

            for backup in backups:
                table.add_row(
                    backup.name,
                    backup.timestamp.strftime("%Y-%m-%d %H:%M:%S"),
                    backup.size or "Unknown",
                )

            console.print(table)

    except ExecutorError as e:
        console.print(f"[red]Executor Error: {e}[/red]")
        raise typer.Exit(1)


@list_app.command("logical")
def list_logical():
    """List available logical backups."""
    check_config()

    try:
        with get_executor() as executor:
            logical_manager = LogicalBackupManager(config)
            backups = logical_manager.list_backups(executor)

            if not backups:
                console.print("[yellow]No logical backups found.[/yellow]")
                return

            table = Table(title="Logical Backups", box=box.ROUNDED)
            table.add_column("Name", style="cyan")
            table.add_column("Database", style="blue")
            table.add_column("Timestamp", style="green")

            for backup in backups:
                table.add_row(
                    backup.name,
                    backup.database or "Unknown",
                    backup.timestamp.strftime("%Y-%m-%d %H:%M:%S"),
                )

            console.print(table)

    except ExecutorError as e:
        console.print(f"[red]Executor Error: {e}[/red]")
        raise typer.Exit(1)


# Status command
@app.command("status")
def status():
    """Show system status."""
    check_config()

    try:
        with get_executor() as executor:
            console.print("[bold blue]DuplicAid Status[/bold blue]\n")

            # Check container status
            postgres_running = executor.check_container_running(
                config.postgres_container
            )
            backup_running = executor.check_container_running(config.backup_container)

            table = Table(title="Container Status", box=box.ROUNDED)
            table.add_column("Container", style="cyan")
            table.add_column("Status", style="green")

            postgres_status = (
                "[green]Running[/green]" if postgres_running else "[red]Stopped[/red]"
            )
            backup_status = (
                "[green]Running[/green]" if backup_running else "[red]Stopped[/red]"
            )

            table.add_row(config.postgres_container, postgres_status)
            table.add_row(config.backup_container, backup_status)

            console.print(table)

            # Show database status if postgres is running
            if postgres_running:
                discovery = DatabaseDiscovery(config)
                databases = discovery.get_databases(executor)

                if databases:
                    console.print("\n")
                    db_table = Table(title="Database Status", box=box.ROUNDED)
                    db_table.add_column("Database", style="cyan")
                    db_table.add_column("Size", style="green")

                    for db_info in databases:
                        db_table.add_row(
                            db_info["name"], db_info.get("size", "Unknown")
                        )

                    console.print(db_table)

    except ExecutorError as e:
        console.print(f"[red]Executor Error: {e}[/red]")
        raise typer.Exit(1)


if __name__ == "__main__":
    app()
